/**
 * ...
 * @author DeathTiger
 */
import com.GameInterface.CharacterCreation.CharacterCreation;
import com.GameInterface.Game.Character;
import com.GameInterface.DistributedValue;
import com.Utils.ID32;
import com.Utils.GlobalSignal;
import com.Utils.Signal;
import com.GameInterface.GUIUtils.XmlParser;
import com.Utils.Archive;
import com.Utils.LDBFormat;

class x.CombatTracker
{
	private var IdleStart:Date;
	private var StartTime:Date;
	private var TotalDamage:Number;
	private var DirectDamage:Number;
	private var PreviousDamage:Number;
	private var ElapsedTime:Number;
	private var ResetTime:Number;
	private var IdleResetTime:Number;
	private var TargetDefensiveCharacter:Character;
	private var TargetOffensiveCharacter:Character;
	private var TargetCharacter:Character;
	private var Player:Character;
    public var SignalTrackerReset:Signal;
    public var SignalTrackerUpdate:Signal;

	public function CombatTracker(charID:ID32, resetTime:Number)
    {	

		Player = Character.GetClientCharacter(charID);
		
		//Tracker Reset Duration
		if (resetTime == undefined) resetTime = 10;
		ResetTime = resetTime;

		SignalTrackerReset = new Signal();
		SignalTrackerUpdate = new Signal();
		
		//Hook the Signal
		GlobalSignal.SignalDamageNumberInfo.Connect( SlotDamageInfo, this );
		Setup();
		setInterval(this, "SlotUpdateTracker", 100);		
    }
   
	private function Setup():Void {
		TargetDefensiveCharacter = undefined;
		TargetOffensiveCharacter = undefined;
		TargetCharacter = undefined;
		TotalDamage = 0;
		DirectDamage = 0;
		ElapsedTime = 0;
		PreviousDamage = 0;
		StartTime = undefined;
		IdleStart = undefined;
		IdleResetTime = 0;
		SignalTrackerReset.Emit();
	}
	
	private function SlotUpdateTracker() {
		if (IdleStart) {
			IdleResetTime = (new Date().getTime() - IdleStart.getTime())/1000;
			if (IdleResetTime > ResetTime) 
				Reset();
		}
	}
	
	private function SlotUpdateDPS() {
		if (TotalDamage > PreviousDamage)
		{
			PreviousDamage = TotalDamage;
			SignalTrackerUpdate.Emit();
		}
	}
	
	public function Reset():Void {
		Setup();
	}
	
	private function SlotDamageInfo(statID:Number, damage:Number, absorb:Number, attackResultType:Number, attackType:Number,  attackOffensiveLevel:Number, attackDefensiveLevel:Number, context:Number, targetID:ID32, iconID:ID32, iconColorLine:Number, combatLogFeedbackType:Number) {
		if (!Player.IsInCombat()) return;
		if (!StartTime) StartTime = new Date();
		if (!IdleStart) IdleStart = new Date();
		var now:Date = new Date();	
		
		ElapsedTime = (now.getTime() - StartTime.getTime())/1000;
		var timeout:Number = (now.getTime() - IdleStart.getTime())/1000;

		var timeout:Number = now.getUTCSeconds() - IdleStart.getUTCSeconds();
		if ( timeout > ResetTime || ElapsedTime < 0) Reset();		
		
		//Show only the interesting ones
		var isHeal:Boolean = attackResultType == _global.Enums.AttackResultType.e_AttackType_SpellHeal || attackResultType == _global.Enums.AttackResultType.e_AttackType_SpellCriticalHeal;
		if (!isHeal && damage != 0 || absorb != 0)
		{
			//target.Defencive = Player Target
			//target.Offensive = Player
			TargetCharacter = Character.GetCharacter(targetID);

			//Don't include healing
			if (TargetCharacter != undefined && !TargetCharacter.GetID().Equal(Player.GetID()))
			{
				TargetDefensiveCharacter = Character.GetCharacter(TargetCharacter.GetDefensiveTarget());
				TargetOffensiveCharacter = Character.GetCharacter(TargetCharacter.GetOffensiveTarget());
				var PlayerDefensiveTarget:Character = Character.GetCharacter(Player.GetDefensiveTarget());
				//Adjust the total DPS
				TotalDamage += damage;
								
				//Adjust the direct DPS
				if (Player.GetID().toString() == TargetCharacter.GetID().toString()) 
				DirectDamage += damage;
				
				if (ElapsedTime == 0) ElapsedTime = 1;				
				IdleStart = new Date();
				SignalTrackerUpdate.Emit();
			}
		}
	}
	
	//Gets the Total DPS for the Player
	public function GetTotalDPS() : Number { 
		if (ElapsedTime == 0) return TotalDamage;
		else return Math.floor(TotalDamage / ElapsedTime);
	}
	//Gets the Total Damage for the Player
	public function GetTotalDamage() : Number {return TotalDamage;}
	//Gets the DirectTargetDamage for the player
	public function GetDirectDPS() : Number {
		if (ElapsedTime == 0) return DirectDamage;
		else return Math.floor(DirectDamage / ElapsedTime); 
	}
	
	public function SetResetTime(seconds:Number) {
		if (seconds > 0)
		ResetTime = seconds;		
	}
	
	//Gets the Elapsed Time
	public function GetDuration() : Number { return Math.round(ElapsedTime); }
	
	public function GetIdle() : Number { return IdleResetTime; }

	public function testXMLNodes():String { return "Not Set";}
	
	//Gets the Player
	public function GetPlayer() : Character {return Player;}
	
	public function GetPlayerTarget() : Character {return TargetCharacter;}
	
	public function GetPlayerTargetOffensive() : Character {return TargetOffensiveCharacter;}
	
	public function GetPlayerTargetDefensive() : Character {return TargetDefensiveCharacter;}
}